@@ -477,6 +477,8 @@ void command_all(gpointer callback_data,guint callback_action,
 	top=ptw->top;
 	sx=MAX(ptw->drawing->allocation.width/ptw->fontsize,1);
 	sy=MAX(ptw->drawing->allocation.height/(ptw->fontsize*2),1);
+	if (ptw->select.x<0)/* 򤹤 */
+		gtk_selection_owner_set(window,GDK_SELECTION_PRIMARY,GDK_CURRENT_TIME);
 	ptw->select.x=ptw->select.y=0;
 	ptw->cursor.x=get_width(&ptw->start,&ptw->off,
 											ptw->cursor.y=ptw->max-1,ptw->tab);
