@@ -16,6 +16,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
+#include "charset.h"
 #include "edit.h"
 #include "other.h"
 
@@ -40,7 +41,7 @@ LINEBUF *get_line_buf(LINEBUF **start,gint *off,gint y)
 		*start=(*start)->next;
 		(*off)++;
 	}
-	return *start;
+	return *off==y?*start:NULL;
 }
 
 
@@ -61,11 +62,13 @@ gint get_width(LINEBUF **start,gint *off,gint y,gint tab)
 			if (p->text[datapos]=='\t') {
 				width=(width/tab+1)*tab;
 				datapos++;
-			} else if (iskanji(p->text[datapos])) {
-				if (datapos==p->length-2)
+			} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
+																<=p->length) {
+				if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
+																==p->length)
 					break;
-				width+=2;
-				datapos+=2;
+				width+=charset->width[(gint)(guchar)p->text[datapos]];
+				datapos+=charset->length[(gint)(guchar)p->text[datapos]];
 			} else {
 				width++;
 				datapos++;
@@ -75,12 +78,13 @@ gint get_width(LINEBUF **start,gint *off,gint y,gint tab)
 			if (p->text[datapos]=='\t') {
 				width=(width/tab+1)*tab;
 				datapos++;
-			} else if (datapos==p->length-1 || !iskanji(p->text[datapos])) {
+			} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
+																<=p->length) {
+				width+=charset->width[(gint)(guchar)p->text[datapos]];
+				datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+			} else {
 				width++;
 				datapos++;
-			} else {
-				width+=2;
-				datapos+=2;
 			}
 	return width;
 }
@@ -98,11 +102,19 @@ gint get_width_max(TEXTWND *ptw)
 								y<ptw->drawing->allocation.height && p!=NULL;
 												y+=ptw->fontsize*2,p=p->next) {
 		width=0;
-		for (datapos=0;datapos<p->length;datapos++)
-			if (p->text[datapos]=='\t')
+		datapos=0;
+		while (datapos<p->length)
+			if (p->text[datapos]=='\t') {
 				width=(width/ptw->tab+1)*ptw->tab;
-			else
+				datapos++;
+			} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
+																>p->length) {
+				width+=charset->width[(gint)(guchar)p->text[datapos]];
+				datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+			} else {
 				width++;
+				datapos++;
+			}
 		if (width>max)
 			max=width;
 	}
@@ -132,16 +144,21 @@ gint get_align_pos(LINEBUF **start,gint *off,
 					return align?(screenpos/tab+1)*tab:screenpos;
 				screenpos=(screenpos/tab+1)*tab;
 				datapos++;
-			} else if (datapos==p->length-1 || !iskanji(p->text[datapos])) {
+			} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
+																>p->length) {
 				screenpos++;
 				datapos++;
-			} else if (datapos==p->length-2) {
+			} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
+																==p->length) {
 				break;
-			} else if (x<screenpos+2) {
-				return align?screenpos+2:screenpos;
+			} else if (x<screenpos
+							+charset->width[(gint)(guchar)p->text[datapos]]) {
+				return align
+					?screenpos+charset->width[(gint)(guchar)p->text[datapos]]
+																	:screenpos;
 			} else {
-				screenpos+=2;
-				datapos+=2;
+				screenpos+=charset->width[(gint)(guchar)p->text[datapos]];
+				datapos+=charset->length[(gint)(guchar)p->text[datapos]];
 			}
 	else
 		while (screenpos<x && datapos<p->length)
@@ -150,14 +167,18 @@ gint get_align_pos(LINEBUF **start,gint *off,
 					return align?(screenpos/tab+1)*tab:screenpos;
 				screenpos=(screenpos/tab+1)*tab;
 				datapos++;
-			} else if (datapos==p->length-1 || !iskanji(p->text[datapos])) {
+			} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
+																>p->length) {
 				screenpos++;
 				datapos++;
-			} else if (x<screenpos+2) {
-				return align?screenpos+2:screenpos;
+			} else if (x<screenpos
+							+charset->width[(gint)(guchar)p->text[datapos]]) {
+				return align
+					?screenpos+charset->width[(gint)(guchar)p->text[datapos]]
+																	:screenpos;
 			} else {
-				screenpos+=2;
-				datapos+=2;
+				screenpos+=charset->width[(gint)(guchar)p->text[datapos]];
+				datapos+=charset->length[(gint)(guchar)p->text[datapos]];
 			}
 	return screenpos;
 }
@@ -184,14 +205,18 @@ gint get_data_pos(LINEBUF **start,gint *off,gint x,gint y,
 			if (x<screenpos)
 				return align?datapos+1:datapos;
 			datapos++;
-		} else if (datapos==p->length-1 || !iskanji(p->text[datapos])) {
+		} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
+																>p->length) {
 			screenpos++;
 			datapos++;
-		} else if (x<screenpos+2) {
-			return align?datapos+2:datapos;
+		} else if (x<screenpos
+							+charset->width[(gint)(guchar)p->text[datapos]]) {
+			return align
+					?datapos+charset->length[(gint)(guchar)p->text[datapos]]
+																	:datapos;
 		} else {
-			screenpos+=2;
-			datapos+=2;
+			screenpos+=charset->width[(gint)(guchar)p->text[datapos]];
+			datapos+=charset->length[(gint)(guchar)p->text[datapos]];
 		}
 	return MIN(datapos,p->length);
 }
@@ -206,15 +231,20 @@ gint get_data_pos(LINEBUF **start,gint *off,gint x,gint y,
 	  RET,ɸ																*/
 gint get_screen_pos(LINEBUF **start,gint *off,gint x,gint y,gint tab)
 {
-	gint datapos,screenpos=0;
+	gint datapos=0,screenpos=0;
 	LINEBUF *p;
 
 	p=get_line_buf(start,off,y);
-	for (datapos=0;datapos<x;datapos++)
-		if (p->text[datapos]=='\t')
+	while (datapos<x)
+		if (p->text[datapos]=='\t') {
 			screenpos=(screenpos/tab+1)*tab;
-		else
-			screenpos++;
+			datapos++;
+		} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]>x) {
+			break;
+		} else {
+			screenpos+=charset->width[(gint)(guchar)p->text[datapos]];
+			datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+		}
 	return screenpos;
 }
 
@@ -226,6 +256,8 @@ gint get_screen_pos(LINEBUF **start,gint *off,gint x,gint y,gint tab)
 guint get_char_type(gchar *text,gint length)
 {
 	if (length>1)
+		return 8;
+	if ((*text&0x80)!=0)
 		return 4;
 	if (('A'<=*text && *text<='Z') || ('a'<=*text && *text<='z'))
 		return 2;
@@ -246,7 +278,7 @@ guint get_char_type(gchar *text,gint length)
 gint get_move_pos(LINEBUF **start,gint *off,gint x,gint y,
 														gint tab,gboolean move)
 {
-	gint i,datapos,length,*type;/* 0:1Хʸ,1:1Х,2Х */
+	gint i,j,datapos,length,*type;/* 1ʸΥХȿ */
 	guint ct,ctype;
 	LINEBUF *p;
 
@@ -254,11 +286,14 @@ gint get_move_pos(LINEBUF **start,gint *off,gint x,gint y,
 	if (move) {
 		/* ذư */
 		datapos=get_data_pos(start,off,x,y,tab,FALSE);
-		length=datapos<p->length-1 && iskanji(p->text[datapos])?2:1;
+		length=datapos+charset->length[(gint)(guchar)p->text[datapos]]
+																	<=p->length
+						?charset->length[(gint)(guchar)p->text[datapos]]:1;
 		ctype=get_char_type(p->text+datapos,length);
 		datapos+=length;
 		while (datapos<p->length) {
-			length=datapos<p->length-1 && iskanji(p->text[datapos])?2:1;
+			length=datapos+charset->length[(gint)(guchar)p->text[datapos]]
+				<=p->length?charset->length[(gint)(guchar)p->text[datapos]]:1;
 			ct=get_char_type(p->text+datapos,length);
 			if ((ctype&ct)==0) {
 				if (ct!=0)
@@ -275,19 +310,17 @@ gint get_move_pos(LINEBUF **start,gint *off,gint x,gint y,
 		type=g_malloc(datapos*sizeof(gint));
 		i=0;
 		while (i<datapos)
-			if (i+1<datapos && iskanji(p->text[i])) {
-				type[i++]=1;
-				type[i++]=2;
+			if (i+charset->length[(gint)(guchar)p->text[i]]<=datapos) {
+				for (j=0;j<charset->length[(gint)(guchar)p->text[i]];j++)
+					type[i+j]=j+1;
+				i+=charset->length[(gint)(guchar)p->text[i]];
 			} else {
-				type[i++]=0;
+				type[i++]=1;
 			}
-		length=type[datapos-1]>0?2:1;
+		length=type[datapos-1];
 		datapos-=length;
 		ctype=get_char_type(p->text+datapos,length);
-		while (--datapos>=0) {
-			if (type[datapos]>1)
-				continue;
-			length=type[datapos]>0?2:1;
+		while (datapos>0 && (datapos-=length=type[datapos-1])>0) {
 			ct=get_char_type(p->text+datapos,length);
 			if ((ctype&ct)==0) {
 				if (ctype!=0) {
@@ -442,16 +475,17 @@ gint put_mem(LINEBUF **start,gint *off,GdkPoint *put,GdkPoint *new,
 					if (margin<screenpos)
 						break;
 					datapos++;
-				} else if (datapos==length-1 || !iskanji(text[datapos])) {
+				} else if (datapos
+					+charset->length[(gint)(guchar)text[datapos]]<=length) {
+					screenpos+=charset->width[(gint)(guchar)text[datapos]];
+					if (margin<screenpos)
+						break;
+					datapos+=charset->length[(gint)(guchar)text[datapos]];
+				} else {
 					screenpos++;
 					if (margin<screenpos)
 						break;
 					datapos++;
-				} else {
-					screenpos+=2;
-					if (margin<screenpos)
-						break;
-					datapos+=2;
 				}
 			if (p->length>0 || i!=datapos)
 				p->text=g_realloc(p->text,(p->length+datapos-i)*sizeof(gchar));
@@ -511,14 +545,22 @@ gint put_mem(LINEBUF **start,gint *off,GdkPoint *put,GdkPoint *new,
 	}
 	if (new!=NULL) {
 		new->x=0;
-		for (datapos=0;datapos<p->length;datapos++)
-			if (p->text[datapos]=='\t')
+		datapos=0;
+		while (datapos<p->length)
+			if (p->text[datapos]=='\t') {
 				new->x=(new->x/tab+1)*tab;
-			else
+				datapos++;
+			} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
+																	<=length) {
+				new->x+=charset->width[(gint)(guchar)p->text[datapos]];
+				datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+			} else {
 				new->x++;
+				datapos++;
+			}
 		new->y=put->y+insert;
-		if (margin>0 && p->margin && (margin<=new->x
-					|| (temp!=NULL && margin-1<=new->x && iskanji(temp[0])))) {
+		if (margin>0 && p->margin && (margin<=new->x || (temp!=NULL
+				&& margin<=new->x+charset->width[(gint)(guchar)temp[0]]))) {
 			new->x=0;
 			new->y++;
 		}
@@ -540,17 +582,17 @@ gint put_mem(LINEBUF **start,gint *off,GdkPoint *put,GdkPoint *new,
 					if (margin<screenpos)
 						break;
 					datapos++;
-				} else if (datapos==p->length-1
-											|| !iskanji(p->text[datapos])) {
+				} else if (datapos
+					+charset->length[(gint)(guchar)text[datapos]]<=length) {
+					screenpos+=charset->width[(gint)(guchar)text[datapos]];
+					if (margin<screenpos)
+						break;
+					datapos+=charset->length[(gint)(guchar)text[datapos]];
+				} else {
 					screenpos++;
 					if (margin<screenpos)
 						break;
 					datapos++;
-				} else {
-					screenpos+=2;
-					if (margin<screenpos)
-						break;
-					datapos+=2;
 				}
 			if (margin<screenpos) {
 				/* ޡĶƤȤԤ */
@@ -649,17 +691,19 @@ gint del_sel_mem(LINEBUF **start,gint *off,
 				if (q->text[datapos]=='\t') {
 					screenpos=(screenpos/tab+1)*tab;
 					datapos++;
-				} else if (iskanji(q->text[datapos])) {
-					screenpos+=2;
-					datapos+=2;
+				} else if (datapos
+							+charset->length[(gint)(guchar)q->text[datapos]]
+																<=q->length) {
+					screenpos+=charset->width[(gint)(guchar)q->text[datapos]];
+					datapos+=charset->length[(gint)(guchar)q->text[datapos]];
 				} else {
 					screenpos++;
 					datapos++;
 				}
 			if (p->text[0]=='\t')
 				screenpos=(screenpos/tab+1)*tab;
-			else if (iskanji(p->text[0]))
-				screenpos+=2;
+			else if (charset->length[(gint)(guchar)p->text[0]]<=p->length)
+				screenpos+=charset->width[(gint)(guchar)p->text[0]];
 			else
 				screenpos++;
 		} else {
@@ -699,17 +743,18 @@ gint del_sel_mem(LINEBUF **start,gint *off,
 					if (margin<screenpos)
 						break;
 					datapos++;
-				} else if (datapos==p->length-1
-											|| !iskanji(p->text[datapos])) {
+				} else if (datapos
+							+charset->length[(gint)(guchar)p->text[datapos]]
+																<=p->length) {
+					screenpos+=charset->width[(gint)(guchar)p->text[datapos]];
+					if (margin<screenpos)
+						break;
+					datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+				} else {
 					screenpos++;
 					if (margin<screenpos)
 						break;
 					datapos++;
-				} else {
-					screenpos+=2;
-					if (margin<screenpos)
-						break;
-					datapos+=2;
 				}
 			if (margin<screenpos) {
 				/* ޡĶƤȤԤ */
