@@ -241,7 +241,7 @@ void open_text_file(TEXTWND *ptw)
 TEXTWND *open_edit_file(gchar *file)
 {
 	gchar *ext,*p,*label;
-	gint i,leng0,leng1,fontsize,lbearing,rbearing,width,ascent,descent;
+	gint i,leng0,leng1,fontsize,ascent;
 	TEXTWND *ptw;
 	GtkWidget *table,*sub_menu;
 	static gint fcount=0;	/* եο */
@@ -327,11 +327,11 @@ TEXTWND *open_edit_file(gchar *file)
 	label=add_edit_file(ptw->file,&ptw->same);
 
 	/* ե */
-	ptw->font=gdk_fontset_load(ptw->fontname);
+	ptw->font=ptw->fontname==NULL?gdk_font_ref(system_font)
+											:gdk_fontset_load(ptw->fontname);
 	ptw->fontascent=ptw->fontsize=0;
 	for (i=0x20;i<=0x7e;i++) {
-		gdk_text_extents(ptw->font,(gchar *)&i,1,
-								&lbearing,&rbearing,&width,&ascent,&descent);
+		gdk_text_extents(ptw->font,(gchar *)&i,1,NULL,NULL,NULL,&ascent,NULL);
 		if (ptw->fontascent<ascent)
 			ptw->fontascent=ascent;
 		fontsize=(gdk_char_height(ptw->font,i)+1)/2;
