@@ -19,6 +19,7 @@
 #include <string.h>
 #include <gdk/gdkkeysyms.h>
 #include "abort.h"
+#include "charset.h"
 #include "edit.h"
 #include "find.h"
 #include "general.h"
@@ -193,19 +194,20 @@ gboolean find_dialog(FIND *fd)
 *                                                                             *
 ******************************************************************************/
 /*	ʸμ
-	  type,ʸ°ǼХåե,0:ANK,1:EUC1Х,2:EUC2Х
+	  type,ʸ°ǼХåե
 	  text,ʸ
 	length,ǡ													*/
 static void make_char_type(gint *type,gchar *text,gint length)
 {
-	gint i=0;
+	gint i=0,j;
 
 	while (i<length)
-		if (iskanji(text[i])) {
-			type[i++]=1;
-			type[i++]=2;
+		if (i+charset->length[(gint)(guchar)text[i]]<=length) {
+			for (j=0;j<charset->length[(gint)(guchar)text[i]];j++)
+				type[i+j]=j+1;
+			i+=charset->length[(gint)(guchar)text[i]];
 		} else {
-			type[i++]=0;
+			type[i++]=1;
 		}
 }
 
@@ -222,7 +224,7 @@ void find_operation(TEXTWND *ptw,GdkPoint *start,GdkPoint *end,
 {
 	gboolean result;			/* Ӥη,TRUE:Ʊ,FALSE:ۤʤ */
 	gchar *text;
-	gint i=0,sx,sy;
+	gint i,max=0,sx,sy;
 	gint length0,length1=0;		/* ʸ(Хñ) */
 	gint *buf=NULL,*type=NULL;
 	LINEBUF *p,*q;
@@ -235,7 +237,11 @@ void find_operation(TEXTWND *ptw,GdkPoint *start,GdkPoint *end,
 	ptw->cursor.x=get_align_pos(&ptw->start,&ptw->off,
 								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
 	length0=strlen(find);
-	text=g_malloc((length0+2)*sizeof(gchar));
+	for (i=0;i<256;i++)
+		if (charset->length[i]>max)
+			max=charset->length[i];
+	text=g_malloc((length0+max)*sizeof(gchar));
+	buf=g_malloc((length0+max)*sizeof(gint));
 	if (ptw->select.x>=0) {
 		/* ϰϤ */
 		select=ptw->select;
@@ -285,12 +291,23 @@ void find_operation(TEXTWND *ptw,GdkPoint *start,GdkPoint *end,
 						&& (data.y<ed.y || (data.y==ed.y && data.x<ed.x))) {
 				if (data.x<q->length) {
 					/* ʸ */
-					if (data.x<q->length-1 && iskanji(q->text[data.x])) {
-						memcpy(text+length1,q->text+data.x,sizeof(gchar)*2);
-						length1+=2;
-						data.x+=2;
+					if (data.x+charset->length[(gint)(guchar)q->text[data.x]]
+																<=q->length) {
+						memcpy(text+length1,q->text+data.x,
+								charset->length[(gint)(guchar)q->text[data.x]]
+															*sizeof(gchar));
+						for (i=0;
+							i<charset->length[(gint)(guchar)q->text[data.x]];
+																		i++)
+							buf[i+length1]=charset->length[(gint)(guchar)
+															q->text[data.x]];
+						length1
+							+=charset->length[(gint)(guchar)q->text[data.x]];
+						data.x
+							+=charset->length[(gint)(guchar)q->text[data.x]];
 					} else {
 						text[length1]=q->text[data.x];
+						buf[length1]=1;
 						length1++;
 						data.x++;
 					}
@@ -302,6 +319,7 @@ void find_operation(TEXTWND *ptw,GdkPoint *start,GdkPoint *end,
 						goto loop;
 					if (!q->margin) {
 						text[length1]='\n';
+						buf[length1]=1;
 						length1++;
 					}
 					data.x=0;
@@ -336,27 +354,16 @@ void find_operation(TEXTWND *ptw,GdkPoint *start,GdkPoint *end,
 				break;
 			}
 			/* ХåեƬʸ˴ */
-			if (iskanji(text[0]) && length1>=2) {
-				memmove(text,text+2,length0*sizeof(gchar));
-				length1-=2;
-				if (st.x<p->length) {
-					st.x+=2;
-				} else {
-					st.x=2;
-					st.y++;
-					p=p->next;
-				}
-			} else if (length1>0) {
-				memmove(text,text+1,(length0+1)*sizeof(gchar));
-				length1--;
-				if (st.x<p->length) {
-					st.x++;
-				} else {
-					st.x=p->margin?1:0;
-					st.y++;
-					p=p->next;
-				}
+			if (st.x<p->length) {
+				st.x+=buf[0];
+			} else {
+				st.x=p->margin?buf[0]:0;
+				st.y++;
+				p=p->next;
 			}
+			length1-=buf[0];
+			memmove(text,text+buf[0],length1*sizeof(gchar));
+			memmove(buf,buf+buf[0],length1*sizeof(gint));
 		}
 	} else {
 		/*  */
@@ -364,29 +371,20 @@ void find_operation(TEXTWND *ptw,GdkPoint *start,GdkPoint *end,
 			type=g_malloc(p->length*sizeof(gint));
 			make_char_type(type,p->text,p->length);
 		}
-		buf=g_malloc(length0*sizeof(gint));
 		while (ed.y<st.y || (st.y==ed.y && ed.x<st.x)) {
 			/* Хåեκǽʸä */
 			while (length1<length0
 						&& (ed.y<data.y || (data.y==ed.y && ed.x<data.x))) {
 				if (0<data.x) {
 					/* ʸ */
-					if (type[data.x-1]==2) {
-						memmove(text+2,text,(length0-2)*sizeof(gchar));
-						memmove(buf+2,buf,(length0-2)*sizeof(gint));
-						memcpy(text,q->text+data.x-2,sizeof(gchar)*2);
-						buf[0]=1;
-						buf[1]=2;
-						length1+=2;
-						data.x-=2;
-					} else {
-						memmove(text+1,text,(length0-1)*sizeof(gchar));
-						memmove(buf+1,buf,(length0-1)*sizeof(gint));
-						text[0]=q->text[data.x-1];
-						buf[0]=0;
-						length1++;
-						data.x--;
-					}
+					memmove(text+type[data.x-1],text,length1*sizeof(gchar));
+					memmove(buf+type[data.x-1],buf,length1*sizeof(gint));
+					memcpy(text,q->text+data.x-type[data.x-1],
+												type[data.x-1]*sizeof(gchar));
+					for (i=0;i<type[data.x-1];i++)
+						buf[i]=i+1;
+					length1+=type[data.x-1];
+					data.x-=type[data.x-1];
 				} else {
 					/*  */
 					while (gtk_events_pending())
@@ -397,10 +395,10 @@ void find_operation(TEXTWND *ptw,GdkPoint *start,GdkPoint *end,
 					type=g_realloc(type,q->length*sizeof(gint));
 					make_char_type(type,q->text,q->length);
 					if (!q->margin) {
-						memmove(text+1,text,(length0-1)*sizeof(gchar));
-						memmove(buf+1,buf,(length0-1)*sizeof(gint));
+						memmove(text+1,text,length1*sizeof(gchar));
+						memmove(buf+1,buf,length1*sizeof(gint));
 						text[0]='\n';
-						buf[0]=0;
+						buf[0]=1;
 						length1++;
 					}
 					data.x=q->length;
@@ -434,25 +432,14 @@ void find_operation(TEXTWND *ptw,GdkPoint *start,GdkPoint *end,
 				break;
 			}
 			/* ХåեκǸʸ˴ */
-			if (buf[length1-1]==2) {
-				length1-=2;
-				if (st.x>0) {
-					st.x-=2;
-				} else {
-					p=p->prev;
-					st.x=p->length-2;
-					st.y--;
-				}
+			if (st.x>0) {
+				st.x-=buf[length1-1];
 			} else {
-				length1--;
-				if (st.x>0) {
-					st.x--;
-				} else {
-					p=p->prev;
-					st.x=p->length-(p->margin?1:0);
-					st.y--;
-				}
+				p=p->prev;
+				st.x=p->length-(p->margin?buf[length1-1]:0);
+				st.y--;
 			}
+			length1-=buf[length1-1];
 		}
 	}
 	loop:
@@ -475,5 +462,6 @@ void find_operation(TEXTWND *ptw,GdkPoint *start,GdkPoint *end,
 		move_text_window(ptw,&top);
 		clear_sel(ptw,&ptw->select,&ptw->cursor);
 		draw_caret(ptw,&cursor);
+		gtk_selection_owner_set(window,GDK_SELECTION_PRIMARY,GDK_CURRENT_TIME);
 	}
 }
