@@ -175,9 +175,9 @@ void format_dialog(void)
 											format_dialog_unselect_row,NULL);
 	/* 륦ɥ */
 	scroll=gtk_scrolled_window_new(NULL,NULL);
+	gtk_container_add(GTK_CONTAINER(scroll),clist);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
 										GTK_POLICY_NEVER,GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scroll),clist);
 	/* ܥ */
 	button0=gtk_button_new_with_label(NULL);
 	button1=gtk_button_new_with_label(NULL);
