@@ -106,9 +106,9 @@ gboolean ftype_dialog(gint *pos)
 	gtk_widget_show_all(clist);
 	/* 륦ɥ */
 	scroll=gtk_scrolled_window_new(NULL,NULL);
+	gtk_container_add(GTK_CONTAINER(scroll),clist);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
 										GTK_POLICY_NEVER,GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scroll),clist);
 	/* ܥå */
 	hbox=gtk_hbox_new(FALSE,0);
 	gtk_box_pack_start(GTK_BOX(hbox),scroll,TRUE,TRUE,8);
