@@ -17,6 +17,8 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 #include <ctype.h>
+#include <string.h>
+#include "charset.h"
 #include "command.h"
 #include "edit.h"
 #include "find.h"
@@ -38,16 +40,18 @@ void hide_caret(TEXTWND *ptw)
 	GdkRectangle rc;
 
 	gtk_timeout_remove(timer_id);
-	caret=FALSE;
 	timer_id=0;
-	rc.x=get_align_pos(&ptw->start,&ptw->off,
+	if (caret) {
+		caret=FALSE;
+		rc.x=get_align_pos(&ptw->start,&ptw->off,
 								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
-	rc.y=ptw->cursor.y;
-	rc.x=(rc.x-ptw->top.x)*ptw->fontsize;
-	rc.y=(rc.y-ptw->top.y)*ptw->fontsize*2;
-	rc.width=ptw->fontsize;
-	rc.height=ptw->fontsize*2;
-	gtk_widget_draw(ptw->drawing,&rc);
+		rc.y=ptw->cursor.y;
+		rc.x=(rc.x-ptw->top.x)*ptw->fontsize;
+		rc.y=(rc.y-ptw->top.y)*ptw->fontsize*2;
+		rc.width=ptw->fontsize;
+		rc.height=ptw->fontsize*2;
+		gtk_widget_draw(ptw->drawing,&rc);
+	}
 }
 
 
@@ -342,8 +346,12 @@ DOING *edit_operation(TEXTWND *ptw,gchar *text,gint length,
 			d->cursor=cursor_old;
 			d->select=cursor_new;
 		}
-		if (select)/* 򤹤 */
+		if (select) {
+			/* 򤹤 */
 			ptw->select=d->select;
+			gtk_selection_owner_set(window,GDK_SELECTION_PRIMARY,
+															GDK_CURRENT_TIME);
+		}
 		/* Ԥ걦 */
 		rc.x=(cursor_old.x-ptw->top.x)*ptw->fontsize;
 		rc.y=(cursor_old.y-ptw->top.y)*ptw->fontsize*2;
@@ -436,17 +444,17 @@ void modify_margin(TEXTWND *ptw)
 					if (ptw->margin<screenpos)
 						break;
 					datapos++;
-				} else if (datapos==p->length-1
-											|| !iskanji(p->text[datapos])) {
+				} else if (datapos+charset->length[(gint)(guchar)
+												p->text[datapos]]<=p->length) {
+					screenpos+=charset->width[(gint)(guchar)p->text[datapos]];
+					if (ptw->margin<screenpos)
+						break;
+					datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+				} else {
 					screenpos++;
 					if (ptw->margin<screenpos)
 						break;
 					datapos++;
-				} else {
-					screenpos+=2;
-					if (ptw->margin<screenpos)
-						break;
-					datapos+=2;
 				}
 			if (ptw->margin<screenpos) {
 				/* ޡĶƤȤԤ */
@@ -512,8 +520,7 @@ void modify_margin(TEXTWND *ptw)
 void change_property(TEXTWND *ptw,FTYPE *ft)
 {
 	gboolean modify;
-	gint i,fontsize,lbearing,rbearing,width,ascent,descent;
-	gint max,sx,sy;
+	gint i,max,sx,sy,fontsize,ascent;
 
 	if (ptw->limit!=ft->limit || ptw->tab!=ft->tab
 								|| (ptw->code && ptw->margin!=ft->margin)) {
@@ -547,17 +554,19 @@ void change_property(TEXTWND *ptw,FTYPE *ft)
 		modify_margin(ptw);
 	}
 	/* ե */
-	gdk_font_unref(ptw->font);
-	ptw->font=gdk_fontset_load(ptw->fontname);
-	ptw->fontascent=ptw->fontsize=0;
-	for (i=0x20;i<=0x7e;i++) {
-		gdk_text_extents(ptw->font,(gchar *)&i,1,
-								&lbearing,&rbearing,&width,&ascent,&descent);
-		if (ptw->fontascent<ascent)
-			ptw->fontascent=ascent;
-		fontsize=(gdk_char_height(ptw->font,i)+1)/2;
-		if (ptw->fontsize<fontsize)
-			ptw->fontsize=fontsize;
+	if (ptw->fontname!=NULL) {
+		gdk_font_unref(ptw->font);
+		ptw->font=gdk_fontset_load(ptw->fontname);
+		ptw->fontascent=ptw->fontsize=0;
+		for (i=0x20;i<=0x7e;i++) {
+			gdk_text_extents(ptw->font,(gchar *)&i,1,
+												NULL,NULL,NULL,&ascent,NULL);
+			if (ptw->fontascent<ascent)
+				ptw->fontascent=ascent;
+			fontsize=(gdk_char_height(ptw->font,i)+1)/2;
+			if (ptw->fontsize<fontsize)
+				ptw->fontsize=fontsize;
+		}
 	}
 	/* С */
 	sx=MAX(ptw->drawing->allocation.width/ptw->fontsize,1);
@@ -708,12 +717,13 @@ void tab_operation(TEXTWND *ptw)
 			if (p->text[datapos]=='\t') {
 				screenpos=(screenpos/ptw->tab+1)*ptw->tab;
 				datapos++;
-			} else if (datapos==p->length-1 || !iskanji(p->text[datapos])) {
+			} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
+																<=p->length) {
+				screenpos+=charset->width[(gint)(guchar)p->text[datapos]];
+				datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+			} else {
 				screenpos++;
 				datapos++;
-			} else {
-				screenpos+=2;
-				datapos+=2;
 			}
 		if (screenpos>datapos) {
 			temp=g_malloc(screenpos*sizeof(gchar));
@@ -723,18 +733,16 @@ void tab_operation(TEXTWND *ptw)
 				if (p->text[datapos]=='\t') {
 					screenpos=(screenpos/ptw->tab+1)*ptw->tab;
 					datapos++;
-				} else if (datapos==p->length-1
-											|| !iskanji(p->text[datapos])) {
-					temp[screenpos]=p->text[datapos];
-					screenpos++;
-					datapos++;
+				} else if (datapos+charset->length[(gint)(guchar)
+												p->text[datapos]]<=p->length) {
+					memmove(temp+screenpos,p->text+datapos,
+							charset->length[(gint)(guchar)p->text[datapos]]);
+					screenpos+=charset->width[(gint)(guchar)p->text[datapos]];
+					datapos+=charset->length[(gint)(guchar)p->text[datapos]];
 				} else {
 					temp[screenpos]=p->text[datapos];
 					screenpos++;
 					datapos++;
-					temp[screenpos]=p->text[datapos];
-					screenpos++;
-					datapos++;
 				}
 			g_free(p->text);
 			p->length=screenpos;
