@@ -18,6 +18,7 @@
 */
 #include <string.h>
 #include <gdk/gdkkeysyms.h>
+#include "charset.h"
 #include "edit.h"
 #include "file.h"
 #include "general.h"
@@ -34,13 +35,19 @@
 gboolean signal_timeout(gpointer data)
 {
 	gint page;
+	GdkRectangle rc;
 	TEXTWND *ptw;
 
 	caret=!caret;
 	if ((page=gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)))>=0) {
 		ptw=gtk_object_get_user_data(GTK_OBJECT(
 					gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook),page)));
-		draw_caret(ptw,NULL);
+		rc.x=(get_align_pos(&ptw->start,&ptw->off,ptw->cursor.x,ptw->cursor.y,
+									ptw->tab,FALSE)-ptw->top.x)*ptw->fontsize;
+		rc.y=(ptw->cursor.y-ptw->top.y)*ptw->fontsize*2;
+		rc.width=ptw->fontsize;
+		rc.height=ptw->fontsize*2;
+		gtk_widget_draw(ptw->drawing,&rc);
 	}
 	return TRUE;
 }
@@ -200,7 +207,7 @@ gboolean signal_config(GtkWidget *widget,
 
 gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,TEXTWND *ptw)
 {
-	gboolean select;
+	gboolean select,space=FALSE;
 	gint i,x,y,datapos,length,width;
 	GdkGC *gc;
 	GdkPoint start,end,cursor,pt[4];
@@ -273,15 +280,16 @@ gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,TEXTWND *ptw)
 					x=(((x/ptw->fontsize+ptw->top.x)/ptw->tab+1)
 										*ptw->tab-ptw->top.x)*ptw->fontsize;
 					datapos++;
-				} else if (datapos==p->length-1
-											|| !iskanji(p->text[datapos])) {
+				} else if (datapos+charset->length[(gint)(guchar)
+												p->text[datapos]]<=p->length) {
+					/* Ⱦ/ */
+					x+=ptw->fontsize
+							*charset->width[(gint)(guchar)p->text[datapos]];
+					datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+				} else {
 					/* Ⱦ */
 					x+=ptw->fontsize;
 					datapos++;
-				} else {
-					/*  */
-					x+=ptw->fontsize*2;
-					datapos+=2;
 				}
 				continue;
 			}
@@ -323,7 +331,46 @@ gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,TEXTWND *ptw)
 										ptw->fontsize*2/3,ptw->fontsize*2/3);
 				}
 				datapos++;
-			} else if (datapos==p->length-1 || !iskanji(p->text[datapos])) {
+			} else if (datapos+charset->length[(gint)(guchar)p->text[datapos]]
+																<=p->length) {
+				/* Ⱦ/ */
+				width=ptw->fontsize
+							*charset->width[(gint)(guchar)p->text[datapos]];
+				if (select) {
+					/*  */
+					gdk_gc_set_foreground(gc,
+									(ptw->syscol?system_color:ptw->color)+9);
+					gdk_draw_rectangle(widget->window,gc,TRUE,x,y,
+														width,ptw->fontsize*2);
+					gdk_gc_set_foreground(gc,
+									(ptw->syscol?system_color:ptw->color)+8);
+				} else {
+					gdk_gc_set_foreground(gc,
+									ptw->syscol?system_color:ptw->color);
+				}
+				if (ptw->space && charset->space!=NULL) {
+					space=FALSE;/* ѥڡȽ */
+					for (i=0;charset->space[i]!=0;i+=charset->space[i]+1)
+						if (charset->length[(gint)(guchar)p->text[datapos]]
+															==charset->space[i]
+								&& memcmp(p->text+datapos,charset->space+i+1,
+														charset->space[i])==0)
+							space=TRUE;
+				}
+				if (space) {
+					/* ѥڡ */
+					gdk_gc_set_foreground(gc,
+									(ptw->syscol?system_color:ptw->color)+3);
+					gdk_draw_rectangle(widget->window,gc,TRUE,x+1,y+1,
+													width-1,ptw->fontsize*2-1);
+				} else {
+					/* ʸ */
+					gdk_draw_text(widget->window,ptw->font,gc,
+										x,y+ptw->fontascent,p->text+datapos,
+							charset->length[(gint)(guchar)p->text[datapos]]);
+				}
+				datapos+=charset->length[(gint)(guchar)p->text[datapos]];
+			} else {
 				/* Ⱦ */
 				width=ptw->fontsize;
 				if (select) {
@@ -341,34 +388,6 @@ gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,TEXTWND *ptw)
 				gdk_draw_text(widget->window,ptw->font,gc,
 										x,y+ptw->fontascent,p->text+datapos,1);
 				datapos++;
-			} else {
-				/*  */
-				width=ptw->fontsize*2;
-				if (select) {
-					/*  */
-					gdk_gc_set_foreground(gc,
-									(ptw->syscol?system_color:ptw->color)+9);
-					gdk_draw_rectangle(widget->window,gc,TRUE,x,y,
-														width,ptw->fontsize*2);
-					gdk_gc_set_foreground(gc,
-									(ptw->syscol?system_color:ptw->color)+8);
-				} else {
-					gdk_gc_set_foreground(gc,
-									ptw->syscol?system_color:ptw->color);
-				}
-				if (ptw->space && (guchar)p->text[datapos]==0xa1
-										&& (guchar)p->text[datapos+1]==0xa1) {
-					/* ѥڡ */
-					gdk_gc_set_foreground(gc,
-									(ptw->syscol?system_color:ptw->color)+3);
-					gdk_draw_rectangle(widget->window,gc,TRUE,x+1,y+1,
-													width-1,ptw->fontsize*2-1);
-				} else {
-					/* ʸ */
-					gdk_draw_text(widget->window,ptw->font,gc,
-										x,y+ptw->fontascent,p->text+datapos,2);
-				}
-				datapos+=2;
 			}
 			/* åȤ */
 			if (caret && GTK_WIDGET_HAS_FOCUS(widget)
@@ -462,15 +481,19 @@ gboolean signal_button_press(GtkWidget *widget,
 				if (ptw->select.x>=0
 									&& (!shift || (ptw->cursor.x==ptw->select.x
 										&& ptw->cursor.y==ptw->select.y))) {
+					/*  */
 					ptw->select.x=-1;
 					clear_sel(ptw,&cursor,&select);
 					set_menu_bar(ptw);
 				} else if (shift && (ptw->cursor.x!=cursor.x
 												|| ptw->cursor.y!=cursor.y)) {
 					if (ptw->select.x<0) {
+						/* 򤹤 */
 						ptw->select.x=get_align_pos(&ptw->start,&ptw->off,
 											cursor.x,cursor.y,ptw->tab,FALSE);
 						ptw->select.y=cursor.y;
+						gtk_selection_owner_set(window,GDK_SELECTION_PRIMARY,
+															GDK_CURRENT_TIME);
 					}
 					clear_sel(ptw,&cursor,&ptw->cursor);
 					set_menu_bar(ptw);
@@ -478,6 +501,63 @@ gboolean signal_button_press(GtkWidget *widget,
 				draw_caret(ptw,&cursor);
 			}
 			break;
+		case 2:/* 楯å */
+			{
+				gchar *text=NULL;
+				gint length;
+				GdkPoint cursor,select;
+				DOING *d;
+
+				if (gdk_selection_owner_get(GDK_SELECTION_PRIMARY)
+										==window->window && ptw->select.x>=0) {
+					/* 쥯ΥʡʬΤȤ */
+					/* ϰϤ */
+					length=get_sel_byte(&ptw->start,&ptw->off,
+										&ptw->select,&ptw->cursor,ptw->tab);
+					text=g_malloc((length+1)*sizeof(gchar));
+					cpy_sel_mem(&ptw->start,&ptw->off,
+									&ptw->select,&ptw->cursor,ptw->tab,text);
+					text[length]='\0';
+				}
+				/* åȰư */
+				cursor=ptw->cursor;
+				select=ptw->select;
+				ptw->cursor.x=ptw->top.x+event->x/ptw->fontsize;
+				ptw->cursor.y=ptw->top.y+event->y/(ptw->fontsize*2);
+				if (ptw->cursor.y<0)
+					ptw->cursor.y=0;
+				else if (ptw->cursor.y>ptw->max-1)
+					ptw->cursor.y=ptw->max-1;
+				if (ptw->cursor.x<get_width(&ptw->start,&ptw->off,
+													ptw->cursor.y,ptw->tab))
+					ptw->cursor.x=get_align_pos(&ptw->start,&ptw->off,
+								ptw->cursor.x,ptw->cursor.y,ptw->tab,FALSE);
+				/*  */
+				if (ptw->select.x>=0) {
+					ptw->select.x=-1;
+					clear_sel(ptw,&cursor,&select);
+					if (text==NULL)
+						set_menu_bar(ptw);
+				}
+				/* å */
+				draw_caret(ptw,&cursor);
+				if (text!=NULL) {
+					/* Žդ */
+					d=edit_operation(ptw,text,strlen(text),FALSE,TRUE);
+					d->next=ptw->undo;
+					ptw->undo=d;
+					if (delete_list(&ptw->redo)>0 || d->next==NULL
+																|| select.x<0)
+						set_menu_bar(ptw);
+					ptw->edit=TRUE;
+					g_free(text);
+				} else {
+					/* 쥯ΥʡʬǤϤʤȤ */
+					gtk_selection_convert(window,GDK_SELECTION_PRIMARY,
+												atom_targets,GDK_CURRENT_TIME);
+				}
+			}
+			break;
 		case 3:/* å */
 			gtk_widget_set_sensitive(gtk_item_factory_get_widget(
 				item_factory_popup,_("<main>/᤹(U)")),ptw->undo!=NULL);
@@ -515,7 +595,7 @@ gboolean signal_button_press(GtkWidget *widget,
 		break;
 	case GDK_2BUTTON_PRESS:
 		{
-			gint i,j,datapos,length,ct,ctype,*type;
+			gint i,j,k,datapos,length,ct,ctype,*type;
 			LINEBUF *p;
 			GdkPoint cursor;
 
@@ -535,26 +615,26 @@ gboolean signal_button_press(GtkWidget *widget,
 				type=g_malloc(datapos*sizeof(gint));
 				/* 򳫻ϰ */
 				while (i<datapos)
-					if (i+1<datapos && iskanji(p->text[i])) {
-						type[i++]=1;
-						type[i++]=2;
+					if (i+charset->length[(gint)(guchar)p->text[i]]<=datapos) {
+						for (k=0;k<charset->length[(gint)(guchar)p->text[i]];
+																		k++)
+							type[i+k]=k+1;
+						i+=charset->length[(gint)(guchar)p->text[i]];
 					} else {
-						type[i++]=0;
+						type[i++]=1;
 					}
-				if (datapos<p->length) {
-					length=datapos<p->length-1
-											&& iskanji(p->text[datapos])?2:1;
-					ctype=get_char_type(p->text+datapos,length);
+				if (i<p->length) {
+					length=i+charset->length[(gint)(guchar)p->text[i]]
+																	<=p->length
+								?charset->length[(gint)(guchar)p->text[i]]:1;
+					ctype=get_char_type(p->text+i,length);
 					j+=length;
 				} else {
-					length=type[datapos-1]>0?type[datapos-1]:1;
-					datapos-=length;
-					ctype=get_char_type(p->text+datapos,length);
+					length=type[i-1];
+					i-=length;
+					ctype=get_char_type(p->text+i,length);
 				}
-				for (i=datapos-1;i>=0;i--) {
-					if (type[i]>1)
-						continue;
-					length=type[i]>0?type[i+1]:1;
+				while (i>0 && (i-=(length=type[i-1]))>0) {
 					ct=get_char_type(p->text+i,length);
 					if (ctype!=ct) {
 						i+=length;
@@ -563,13 +643,15 @@ gboolean signal_button_press(GtkWidget *widget,
 				}
 				g_free(type);
 			} else {
-				length=p->length>1 && iskanji(p->text[0])?2:1;
+				length=charset->length[(gint)(guchar)p->text[0]]<=p->length
+								?charset->length[(gint)(guchar)p->text[0]]:1;
 				ctype=get_char_type(p->text,length);
 				j+=length;
 			}
 			/* λ */
 			while (j<p->length) {
-				length=j<p->length-1 && iskanji(p->text[j])?2:1;
+				length=j+charset->length[(gint)(guchar)p->text[j]]
+					<=p->length?charset->length[(gint)(guchar)p->text[j]]:1;
 				ct=get_char_type(p->text+j,length);
 				if (ctype!=ct)
 					break;
@@ -581,11 +663,15 @@ gboolean signal_button_press(GtkWidget *widget,
 													j,ptw->cursor.y,ptw->tab);
 			ptw->select.y=ptw->cursor.y;
 			if (ptw->select.x==ptw->cursor.x) {
+				/*  */
 				ptw->select.x=-1;
 			} else {
+				/* 򤹤 */
 				set_menu_bar(ptw);
 				draw_caret(ptw,&cursor);
 				clear_sel(ptw,&ptw->select,&ptw->cursor);
+				gtk_selection_owner_set(window,GDK_SELECTION_PRIMARY,
+															GDK_CURRENT_TIME);
 			}
 		}
 		break;
@@ -606,9 +692,10 @@ static gboolean signal_motion_notify_draw(TEXTWND *ptw,gint x,gint y,
 														GdkModifierType state)
 {
 	gint s,sx,sy;
-	GdkPoint cursor,top,ps;
+	GdkPoint cursor,select,top,ps;
 
 	cursor=ptw->cursor;
+	select=ptw->select;
 	top=ptw->top;
 	sx=MAX(ptw->drawing->allocation.width/ptw->fontsize,1);
 	sy=MAX(ptw->drawing->allocation.height/(ptw->fontsize*2),1);
@@ -663,6 +750,8 @@ static gboolean signal_motion_notify_draw(TEXTWND *ptw,gint x,gint y,
 	}
 	if (ptw->select.x==ptw->cursor.x && ptw->select.y==ptw->cursor.y)
 		ptw->select.x=-1;/* νȽ꤬ƱȤ */
+	else if (select.x==-1)/* 򤹤 */
+		gtk_selection_owner_set(window,GDK_SELECTION_PRIMARY,GDK_CURRENT_TIME);
 	if (ptw->cursor.x!=cursor.x || ptw->cursor.y!=cursor.y
 								|| ptw->top.x!=top.x || ptw->top.y!=top.y) {
 		set_menu_bar(ptw);
@@ -914,6 +1003,9 @@ gboolean signal_key_press(GtkWidget *widget,GdkEventKey *event,TEXTWND *ptw)
 										&ptw->off,ptw->cursor.x,ptw->cursor.y,
 															ptw->tab,FALSE))
 					ptw->select.x=-1;/* ϰϤƱȤ */
+				else
+					gtk_selection_owner_set(window,GDK_SELECTION_PRIMARY,
+															GDK_CURRENT_TIME);
 			}
 			if (ptw->cursor.x==cursor.x && ptw->cursor.y==cursor.y
 						&& ptw->top.x==top.x && ptw->top.y==top.y
@@ -1166,10 +1258,11 @@ void signal_destroy_draw(GtkWidget *widget,TEXTWND *ptw)
 ******************************************************************************/
 void signal_style_set(GtkWidget *widget,GtkStyle *style,gpointer user_data)
 {
-	gint i;
+	gint i,j,max,sx,sy,fontsize,ascent;
 	GList *glist;
 	TEXTWND *ptw;
 
+	/* ƥ࿧ */
 	style=gtk_widget_get_style(widget);
 	system_color[0]=style->text[0];
 	system_color[1]=style->base[0];
@@ -1181,20 +1274,59 @@ void signal_style_set(GtkWidget *widget,GtkStyle *style,gpointer user_data)
 	system_color[7]=style->mid[0];
 	system_color[8]=style->fg[3];
 	system_color[9]=style->bg[3];
+	if (system_font!=NULL)
+		gdk_font_unref(system_font);
+	/* ƥե */
+	system_font=gdk_font_ref(style->font);
 	glist=gtk_container_children(GTK_CONTAINER(notebook));
 	for (i=g_list_length(glist)-1;i>=0;i--) {
 		ptw=gtk_object_get_user_data(GTK_OBJECT(g_list_nth_data(glist,i)));
-		if (ptw->syscol) {
-			gtk_widget_draw(ptw->drawing,NULL);
+		if (ptw->syscol || ptw->fontname==NULL) {
+			if (ptw->fontname==NULL) {
+				/* եѹ */
+				gdk_font_unref(ptw->font);
+				ptw->font=gdk_font_ref(system_font);
+				ptw->fontascent=ptw->fontsize=0;
+				for (j=0x20;j<=0x7e;j++) {
+					gdk_text_extents(ptw->font,(gchar *)&j,1,
+												NULL,NULL,NULL,&ascent,NULL);
+					if (ptw->fontascent<ascent)
+						ptw->fontascent=ascent;
+					fontsize=(gdk_char_height(ptw->font,j)+1)/2;
+					if (ptw->fontsize<fontsize)
+						ptw->fontsize=fontsize;
+				}
+				/* С */
+				sx=MAX(ptw->drawing->allocation.width/ptw->fontsize,1);
+				sy=MAX(ptw->drawing->allocation.height/(ptw->fontsize*2),1);
+				if (ptw->top.y>ptw->max-sy)
+					ptw->top.y=MAX(ptw->max-sy,0);
+				max=get_width_max(ptw);
+				if (ptw->top.x>max-sx+1)
+					ptw->top.x=MAX(max-sx+1,0);
+				set_scroll_bar(ptw->hscroll,signal_value_changed_hscroll,ptw,
+												0,max,sx,ptw->top.x);
+				set_scroll_bar(ptw->vscroll,signal_value_changed_vscroll,ptw,
+												0,ptw->max-1,sy,ptw->top.y);
+			}
+			gtk_widget_draw(ptw->drawing,NULL);/*  */
 #ifdef USE_XIM
 			if (gdk_im_ready() && ptw->ic!=NULL
 				&& (gdk_ic_get_style(ptw->ic)&GDK_IM_PREEDIT_POSITION)!=0) {
-				gdk_ic_get_attr(ptw->ic,ptw->ic_attr,GDK_IC_PREEDIT_FOREGROUND
-												| GDK_IC_PREEDIT_BACKGROUND);
-				ptw->ic_attr->preedit_foreground=system_color[0];
-				ptw->ic_attr->preedit_background=system_color[1];
-				gdk_ic_set_attr(ptw->ic,ptw->ic_attr,GDK_IC_PREEDIT_FOREGROUND
-												| GDK_IC_PREEDIT_BACKGROUND);
+				gdk_ic_get_attr(ptw->ic,ptw->ic_attr,
+					(ptw->fontname==NULL?GDK_IC_PREEDIT_FONTSET:0)
+					| (ptw->syscol?GDK_IC_PREEDIT_FOREGROUND
+											| GDK_IC_PREEDIT_BACKGROUND:0));
+				if (ptw->syscol) {
+					ptw->ic_attr->preedit_foreground=system_color[0];
+					ptw->ic_attr->preedit_background=system_color[1];
+				}
+				if (ptw->fontname==NULL)
+					ptw->ic_attr->preedit_fontset=ptw->font;
+				gdk_ic_set_attr(ptw->ic,ptw->ic_attr,
+					(ptw->fontname==NULL?GDK_IC_PREEDIT_FONTSET:0)
+					| (ptw->syscol?GDK_IC_PREEDIT_FOREGROUND
+											| GDK_IC_PREEDIT_BACKGROUND:0));
 			}
 #endif
 		}
@@ -1271,9 +1403,7 @@ void signal_drag_data_received(GtkWidget *widget,GdkDragContext *context,
 gboolean signal_selection_clear(GtkWidget *widget,GdkEventSelection *event,
 															gpointer user_data)
 {
-	if (event->selection==GDK_SELECTION_PRIMARY) {
-		has_selection=FALSE;
-	} else if (event->selection==atom_clipboard) {
+	if (event->selection==atom_clipboard) {
 		g_free(clipboard_text);
 		clipboard_text=NULL;
 	}
@@ -1284,11 +1414,14 @@ gboolean signal_selection_clear(GtkWidget *widget,GdkEventSelection *event,
 void signal_selection_get(GtkWidget *widget,GtkSelectionData *data,
 									guint info,guint time,gpointer user_data)
 {
-	gchar *str=NULL;
-	gint length=0,page;
+	gchar *text=NULL;
+	guchar *compound_text;
+	gint length=0,compound_length,format,page;
+	GdkAtom encoding;
 	TEXTWND *ptw;
 
 	if (data->selection==GDK_SELECTION_PRIMARY) {
+		/* ץ饤ޥꥻ쥯 */
 		page=gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
 		if (page>=0) {
 			ptw=gtk_object_get_user_data(GTK_OBJECT(
@@ -1296,40 +1429,36 @@ void signal_selection_get(GtkWidget *widget,GtkSelectionData *data,
 			if (ptw->select.x>=0) {
 				length=get_sel_byte(&ptw->start,&ptw->off,
 										&ptw->select,&ptw->cursor,ptw->tab);
-				str=g_malloc((length+1)*sizeof(gchar));
+				text=g_malloc((length+1)*sizeof(gchar));
 				cpy_sel_mem(&ptw->start,&ptw->off,
-									&ptw->select,&ptw->cursor,ptw->tab,str);
-				str[length]='\0';
+									&ptw->select,&ptw->cursor,ptw->tab,text);
+				text[length]='\0';
 			}
 		}
 	} else if (data->selection==atom_clipboard) {
 		/* åץܡ */
 		if (clipboard_text!=NULL) {
-			str=clipboard_text;
+			text=clipboard_text;
 			length=strlen(clipboard_text);
 		}
 	} else if (data->selection==atom_textmaid) {
 		/* ץ̿ */
-		str="Text maid Selection";
-		length=strlen(str);
+		text="Text maid Selection";
+		length=strlen(text);
 	}
-	if (str!=NULL) {
+	if (text!=NULL) {
 		if (info==TARGET_STRING) {
 			gtk_selection_data_set(data,GDK_SELECTION_TYPE_STRING,
-										8*sizeof(gchar),(guchar *)str,length);
+										8*sizeof(gchar),(guchar *)text,length);
 		} else if (info==TARGET_TEXT || info==TARGET_COMPOUND_TEXT) {
-			guchar *text;
-			GdkAtom encoding;
-			gint format;
-			gint new_length;
-
-			gdk_string_to_compound_text(str,&encoding,&format,
-															&text,&new_length);
-			gtk_selection_data_set(data,encoding,format,text,new_length);
-			gdk_free_compound_text(text);
+			gdk_string_to_compound_text(text,&encoding,&format,
+											&compound_text,&compound_length);
+			gtk_selection_data_set(data,encoding,format,
+												compound_text,compound_length);
+			gdk_free_compound_text(compound_text);
 		}
 		if (data->selection==GDK_SELECTION_PRIMARY)
-			g_free(str);
+			g_free(text);
 	}
 }
 
@@ -1344,18 +1473,22 @@ void signal_selection_received(GtkWidget *widget,GtkSelectionData *data,
 	GdkAtom *atoms;
 	GdkPoint select;
 
-	if (data->selection==atom_clipboard) {
-		/* åץܡ */
+	if (data->selection==GDK_SELECTION_PRIMARY
+										|| data->selection==atom_clipboard) {
+		/* ץ饤ޥꥻ쥯/åץܡ */
 		if (data->length>=0) {
 			if (data->type==GDK_SELECTION_TYPE_ATOM) {
 				atoms=(GdkAtom *)data->data;
 				count=data->length/sizeof(GdkAtom);
 				for (i=0;i<count;i++)
-					if (atoms[i]==GDK_TARGET_STRING
-							|| atoms[i]==atom_compound || atoms[i]==atom_text)
+					if (atoms[i]==atom_compound || atoms[i]==atom_text)
 						break;
+				if (i>=count)
+					for (i=0;i<count;i++)
+						if (atoms[i]==GDK_TARGET_STRING)
+							break;
 				if (i<count)
-					gtk_selection_convert(widget,atom_clipboard,atoms[i],
+					gtk_selection_convert(widget,data->selection,atoms[i],
 															GDK_CURRENT_TIME);
 			} else if (data->type==GDK_TARGET_STRING) {
 				text=g_strdup(data->data);
@@ -1378,7 +1511,8 @@ void signal_selection_received(GtkWidget *widget,GtkSelectionData *data,
 					gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook),
 					gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)))));
 			select=ptw->select;
-			d=edit_operation(ptw,text,strlen(text),TRUE,FALSE);
+			d=edit_operation(ptw,text,strlen(text),
+										data->selection==atom_clipboard,FALSE);
 			d->next=ptw->undo;
 			ptw->undo=d;
 			if (delete_list(&ptw->redo)>0 || d->next==NULL || select.x>=0)
