@@ -21,6 +21,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include "tmaid.h"
+#include "charset.h"
 #include "command.h"
 #include "conf.h"
 #include "file.h"
@@ -38,7 +39,6 @@
 
 gboolean caret=FALSE;				/* åȤη */
 gboolean ins=TRUE;					/* TRUE:,FALSE: */
-gboolean has_selection=FALSE;		/* TRUE:ͭ,FALSE:ʤ */
 gboolean newfile=FALSE;				/* TRUE:ե,FALSE:ʤ */
 gboolean second=FALSE;				/* TRUE:,FALSE:¸ */
 gchar *hisfile[10];					/* ե */
@@ -62,6 +62,7 @@ GtkWidget *status;					/* ơɥ */
 GtkPositionType n_pos=GTK_POS_TOP;	/* Ρȥ֥åΰ */
 GtkItemFactory *item_factory_menu;	/* ˥塼 */
 GtkItemFactory *item_factory_popup;	/* ݥåץå */
+GdkFont *system_font=NULL;
 GdkColor system_color[12]={	{0,0x0000,0x0000,0x0000},{0,0xffff,0xffff,0xffff},
 							{0,0x8000,0x8000,0x8000},{0,0x8000,0x8000,0x8000},
 							{0,0x8000,0x8000,0x8000},{0,0x8000,0x8000,0x8000},
@@ -78,7 +79,7 @@ GdkColor system_color[12]={	{0,0x0000,0x0000,0x0000},{0,0xffff,0xffff,0xffff},
 int main(int argc,char *argv[])
 {
 	gboolean *arg_files;
-	gchar *arg,*endptr,*str,*section=NULL,*key=NULL;
+	gchar *arg,*endptr,*str,*section=NULL,*key=NULL,*lang,**charsetname;
 	gint i,j,k,l,m,opt,leng;
 	gint init_line=0;					/* ΥåȰ */
 	COMM *p;
@@ -169,6 +170,29 @@ int main(int argc,char *argv[])
 	gtk_set_locale();
 	gtk_init(&argc,&argv);
 
+	/* Ķѿ */
+	if ((str=g_getenv("LC_ALL"))!=NULL)
+		while (*str!='\0')
+			if (*str++=='.') {
+				lang=g_strdup(str);
+				for (i=0;lang[i]!='\0';i++)
+					if (lang[i]=='@') {
+						lang[i]='\0';
+						break;
+					}
+				for (i=0;charsetlist[i].name!=NULL;i++) {
+					charsetname=g_strsplit(charsetlist[i].name,",",G_MAXINT);
+					for (j=0;charsetname[j]!=NULL;j++)
+						if (g_strcasecmp(str,charsetname[j])==0) {
+							charset=charsetlist+i;
+							break;
+						}
+					g_strfreev(charsetname);
+				}
+				g_free(lang);
+				break;
+			}
+
 	def_width=gdk_screen_width()*2/3;
 	def_height=gdk_screen_height()*2/3;
 	memset(hisfile,0,sizeof(gchar *)*10);
@@ -319,9 +343,7 @@ int main(int argc,char *argv[])
 			/* ե */
 			if ((ftype[i].fontname
 							=profile_get_string(profile,section,"font"))==NULL)
-				ftype[i].fontname=g_strdup(
-							"-*-*-*-r-normal-*-16-*-*-*-*-*-iso8859-1,"
-							"-*-*-*-r-normal-*-16-*-*-*-*-*-jisx0208.1983-0");
+				ftype[i].fontname=NULL;
 			/*  */
 			if (!profile_get_value(profile,section,"color",
 								(gpointer)&ftype[i].color,sizeof(GdkColor)*12,
@@ -454,8 +476,7 @@ int main(int argc,char *argv[])
 		ftype[0].uline=FALSE;
 		ftype[0].vline=FALSE;
 		/* ե */
-		ftype[0].fontname=g_strdup("-*-*-*-r-normal-*-16-*-*-*-*-*-iso8859-1,"
-							"-*-*-*-r-normal-*-16-*-*-*-*-*-jisx0208.1983-0");
+		ftype[0].fontname=NULL;
 		/*  */
 		memcpy(ftype[0].color,system_color,sizeof(GdkColor)*12);
 	}
@@ -621,6 +642,8 @@ int main(int argc,char *argv[])
 	system_color[7]=style->mid[0];
 	system_color[8]=style->fg[3];
 	system_color[9]=style->bg[3];
+	/* ƥե */
+	system_font=gdk_font_ref(style->font);
 
 	/* ɥ */
 	j=0;
@@ -763,7 +786,8 @@ int main(int argc,char *argv[])
 								(gpointer)&ftype[i].vline,sizeof(gboolean),
 													PROFILE_VALUE_TYPE_BOOL);
 			/* ե */
-			profile_set_value(profile,section,"font",
+			if (ftype[i].fontname!=NULL)
+				profile_set_value(profile,section,"font",
 								(gpointer)ftype[i].fontname,
 								(strlen(ftype[i].fontname)+1)*sizeof(gchar),
 													PROFILE_VALUE_TYPE_STRING);
@@ -779,6 +803,7 @@ int main(int argc,char *argv[])
 		g_free(section);
 		profile_close(profile);
 	}
+	gdk_font_unref(system_font);
 	for (i=0;i<find_num;i++)
 		g_free(find_text[i]);
 	for (i=0;i<replace_num;i++)
