@@ -123,7 +123,6 @@ enum {
 
 extern gboolean caret;					/* åȤη */
 extern gboolean ins;					/* TRUE:,FALSE: */
-extern gboolean has_selection;			/* TRUE:ͭ,FALSE:ʤ */
 extern gboolean newfile;				/* TRUE:ե,FALSE:ʤ */
 extern gboolean second;					/* TRUE:,FALSE:¸ */
 extern gchar *clipboard_text;			/* åץܡɤΥƥ */
@@ -147,6 +146,7 @@ extern GtkWidget *status;				/* ơɥ */
 extern GtkPositionType n_pos;			/* Ρȥ֥åΰ */
 extern GtkItemFactory *item_factory_menu;	/* ˥塼 */
 extern GtkItemFactory *item_factory_popup;	/* ݥåץå */
+extern GdkFont *system_font;
 extern GdkColor system_color[12];
 
 
